#include <ball.h>
#include <game.h>
#include <player.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

struct ball* ball_init(int x_location, int y_location, int width, int speed)
{
    struct ball* ball = malloc(sizeof(struct ball));
    memset(ball, 0, sizeof(struct ball));

    ball->x_location = x_location;
    ball->y_location = y_location;
    ball->width = width;
    ball->speed = speed;
    ball->x_direction = 1.0;
    ball->y_direction = 1.0;

    return ball;
}

static bool check_collision(int x1, int y1, int w1, int h1, int x2, int y2,
                            int w2, int h2)
{
    // Simple AABB collision
    return (x1 < x2 + w2 && x1 + w1 > x2 && y1 < y2 + h2 && y1 + h1 > y2);
}

static void collide_with_player(struct player* player, struct ball* ball)
{
    ball->x_direction = -ball->x_direction;
    // Calculate how much drift this hit should add to the ball
    //
    //   _          player->y_location
    //  ¦ ¦
    //  ¦ ¦         ball->y_location
    //  ¦ ¦ O
    //  ¦ ¦ -----   player->y_location / 2
    //  ¦ ¦
    //  ¦ ¦
    //  ¦_¦
    //
    // The following equation will create a float between -1.0 and 1.0 where
    // 0 means the ball hit the player paddle straight on, -1.0 means the ball
    // hit the topmost point of the paddle and 1.0 means the bottommost point.
    float drift = (float)((ball->y_location + (ball->width / 2)) -
                          (player->y_location + PLAYER_HEIGHT / 2)) /
                  ((float)PLAYER_HEIGHT / 2.0f);
    ball->y_direction += drift;
}

void ball_update(struct game_data* game, struct ball* ball, float dt)
{
    ball->x_location += ball->x_direction * (float) ball->speed * dt;
    ball->y_location += ball->y_direction * (float) ball->speed * dt;

    // Check collisions with the top and bottom of the frame
    if (ball->y_location < 0)
    {
        // collide with top of the frame
        ball->y_direction = -ball->y_direction;
        ball->y_location = 0;
    }
    else if (ball->y_location + ball->width > game->window_height)
    {
        // collide with bottom of the frame
        ball->y_direction = -ball->y_direction;
        ball->y_location = game->window_height - ball->width;
    }
    // Check collisions with the left and right of the frame
    if (ball->x_location < 0)
    {
        // end game on the left
        game->is_running = 0;
    }
    else if (ball->x_location + ball->width > game->window_width)
    {
        // end game on the right
        game->is_running = 0;
    }

    // Check collision with players
    struct player* p1 = game->player1;
    struct player* p2 = game->player2;
    if (check_collision(ball->x_location, ball->y_location, ball->width,
                        ball->width, p1->x_location, p1->y_location,
                        PLAYER_WIDTH, PLAYER_HEIGHT))
    {
        ball->x_location = p1->x_location + PLAYER_WIDTH;
        collide_with_player(p1, ball);
    }
    else if (check_collision(ball->x_location, ball->y_location, ball->width,
                             ball->width, p2->x_location, p2->y_location,
                             PLAYER_WIDTH, PLAYER_HEIGHT))
    {
        ball->x_location = p2->x_location - ball->width;
        collide_with_player(p2, ball);
    }
}